/////////////////////////////////////////////////////////////////////////////////////////
//
// FAT
//
// --------------------------------------------------------------------------------------
//
// Filename:      fat.h
// Version:       1.0
// Date:          15/10/2011
// Author:        Joel Guittet - http://myfreescalewebpage.free.fr
//
/////////////////////////////////////////////////////////////////////////////////////////
//
// Revisions
//
// Version	| Author		| Description
// --------------------------------------------------------------------------------------
//			|				| 
//
/////////////////////////////////////////////////////////////////////////////////////////


#ifndef FAT_H_
#define FAT_H_


//---------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------

#include "typedef.h"
#include "sd.h"


//---------------------------------------------------------------------------------------
// FAT definitions
//---------------------------------------------------------------------------------------

#define SWAP16(int16)						(((int16 & 0x00FF) << 8) + ((int16 & 0xFF00) >> 8))
#define SWAP32(int32)						(((int32 & 0x000000FF) << 24) + ((int32 & 0x0000FF00) << 8) + ((int32 & 0x00FF0000) >> 8) + ((int32 & 0xFF000000) >> 24))

#define FILE_CLEAR          				(0x00)
#define FILE_ERASED         				(0xE5)

#define FILE_AT_READONLY         			(0x01)
#define FILE_AT_HIDDEN           			(0x02)
#define FILE_AT_SYSTEM           			(0x04)
#define FILE_AT_VOLUME           			(0x08)
#define FILE_AT_DIRECTORY        			(0x10)
#define FILE_AT_ARCHIVE          			(0x20)

/* Functions */
typedef enum 
{
    READ									= 0,
    CREATE									= 1,
    MODIFY									= 2,
    NEXT_ENTRY								= 3,
    WRITE_ENTRY								= 4
}
t_FileFunction;

/* Error Codes */
typedef enum 
{
    FILE_FOUND								= 0,
    FILE_NOT_FOUND							= 1,
    FILE_CREATE_OK							= 2,
    NO_FILE_ENTRY_AVAILABLE					= 3,
    NO_FAT_ENTRY_AVAIlABLE					= 4,
	ERROR_IDLE								= 5,
}
t_FileErrorCode;

/* File Handler Structures */
typedef struct
{
    UINT16  FAT_Entry;
    UINT16  SectorOffset;
    UINT16  Dir_Entry;
    UINT32  File_Size;
}
t_ReadFileHandler;

typedef struct
{
    UINT8   FileName[8];
    UINT8   Extension[3];
    UINT16  Dir_Entry;
    UINT32  File_Size;
    UINT16  BaseFatEntry;
    UINT16  CurrentFatEntry;
    UINT16  SectorIndex;
    UINT16  ClusterIndex;
}
t_WriteFileHandler;

/* Root Directory Structure */
typedef struct
{
    UINT8   FileName[8];
    UINT8   Extension[3];
    UINT8   Attributes;
    UINT8   Case;
    UINT8   MiliSeconds;
    UINT16  CreationTime;
    UINT16  CreationDate;
    UINT16  AccessDate;
    UINT16  Reserved;
    UINT16  ModificationTime;
    UINT16  ModificationDate;
    UINT16  ClusterNumber;
    UINT32	SizeofFile;
}
t_RootEntries;

/* Partition Boot Sector Structure */
typedef struct
{
	/* Jump instruction */
    UINT8   JMP_NOP[3];
    /* OEM Name in text */
    UINT8   OEMName[8];
    /* BIOS Parameter Block */
    UINT16  BytesPerSector;
    UINT8   SectorsPerCluster;
    UINT16  ReservedSectors;
    UINT8   FatCopies;
    UINT16  RootDirectoryEntries;
    UINT16  SectorsLess32MB;
    UINT8   MediaDescriptor;
    UINT16  SectorsPerFat;
    UINT16  SectorsPerTrack;
    UINT16  NumberOfHeads;
    UINT32  HiddenSectors;                     
    UINT32  SectorsInPartition;
    /* Extended BIOS Parameter Block */
    UINT16  LogicalNumberOfPartitions;
    UINT8   ExtendedSignature;
    UINT32  SerialNumber;
    UINT8   VolumeNumber[11];
    UINT8   FatName[8];
    /* Bootstrap code */
    UINT8   ExcecutableCode[448];
    /* End of sector marker */
    UINT8   ExcecutableMarker[2];
}
t_PartitionBootSector;


//---------------------------------------------------------------------------------------
// Prototypes
//---------------------------------------------------------------------------------------

void FAT_ReadBootSector(void);
UINT8 FAT_FileOpen(UINT8 * pu8FileName, UINT8 u8Function);
UINT16 FAT_FileRead(UINT8 * pu8UserBuffer);
void FAT_FileWrite(UINT8 * pu8DataPointer, UINT32 u32Size);
void FAT_FileClose(void);
UINT32 FAT_GetFileSize(void);
static void FAT_FileNameOrganizer(UINT8 * pu8FileName, UINT8 * pu8Destiny);
static UINT16 FAT_Entry(UINT16 u16FatEntry, UINT16 u16FatValue, UINT8 u8Function);
static UINT16 FAT_SearchAvailableFAT(UINT16 u16CurrentFAT);


#endif
